package ctbrec.docs;

import java.io.ByteArrayOutputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.vladsch.flexmark.html.HtmlRenderer;
import com.vladsch.flexmark.parser.Parser;
import com.vladsch.flexmark.util.ast.Node;
import com.vladsch.flexmark.util.options.MutableDataSet;

public class MarkdownServlet extends HttpServlet {

    private static final transient Logger LOG = LoggerFactory.getLogger(MarkdownServlet.class);

    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        String path = req.getRequestURI();
        LOG.debug("Path: [{}]", path);
        try {
            String md = loadMarkdown(path);
            String html = markdownToHtml(md);
            resp.setStatus(HttpServletResponse.SC_OK);
            resp.getWriter().println(getHeader());
            resp.getWriter().println(html);
            resp.getWriter().println(getFooter());
        } catch (FileNotFoundException e) {
            resp.setStatus(HttpServletResponse.SC_NOT_FOUND);
            resp.setContentType("text/plain");
            resp.getWriter().println("Hö?");
            resp.getWriter().println();
        }
    }

    private String markdownToHtml(String markdown) {
        MutableDataSet options = new MutableDataSet();
        Parser parser = Parser.builder(options).build();
        HtmlRenderer renderer = HtmlRenderer.builder(options).build();
        Node document = parser.parse(markdown);
        String html = renderer.render(document);
        return html;
    }

    private String loadMarkdown(String path) throws IOException {
        String resource = "/html" + path;
        return loadFile(resource);
    }

    private String loadFile(String resource) throws IOException {
        InputStream resourceAsStream = getClass().getResourceAsStream(resource);
        if(resourceAsStream == null) {
            throw new FileNotFoundException();
        }
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        int length = 0;
        byte[] buffer = new byte[1024];
        while( (length = resourceAsStream.read(buffer)) >= 0 ) {
            out.write(buffer, 0, length);
        }
        return new String(out.toByteArray(), "utf-8");
    }

    private String getHeader() throws IOException {
        return loadFile("/html/docs/header.html");
    }

    private String getFooter() throws IOException {
        return loadFile("/html/docs/footer.html");
    }
}
